Encoding: UTF-8
Package: dplR
Type: Package
Title: Dendrochronology Program Library in R
Version: 1.7.9
Authors@R: c(person("Andy", "Bunn", role = c("aut", "cph","cre", "trl"), email = "bunna@wwu.edu"), 
        person("Mikko","Korpela", role = c("aut", "cph", "trl")), 
        person("Franco", "Biondi", role = c("aut", "cph")), 
        person("Filipe", "Campelo", role = c("aut", "cph")), 
        person("Stefan","Klesse", role = c("aut", "cph")),
        person("Pierre", "Mérian", role = c("aut","cph")), 
        person("Fares", "Qeadan", role = c("aut", "cph")),
        person("Christian", "Zang", role = c("aut", "cph")),
        person("Allan", "Buras", role = "ctb"), 
        person("Alice","Cecile", role = "ctb"), 
        person("Manfred", "Mudelsee", role ="ctb"), 
        person("Michael", "Schulz", role = "ctb"),
        person("David", "Frank",role = "ctb"), 
        person("Ronald","Visser", role = "ctb"),
        person("Ed","Cook", role = "ctb"),
        person("Kevin","Anchukaitis", role = "ctb"))
Copyright: Authors and file inst/COPYRIGHTS
Depends: R (>= 3.5.0)
Imports: graphics, grDevices, grid, stats, utils, lattice (>= 0.13-6),
        Matrix (>= 1.0-3), digest (>= 0.2.3), matrixStats (>= 0.50.2),
        png (>= 0.1-2), R.utils (>= 1.32.1), stringi (>= 0.2-3),
        stringr (>= 0.4), XML (>= 2.1-0), signal, boot, lme4, lifecycle
Suggests: Cairo (>= 1.5-0), dichromat (>= 1.2-3), foreach, forecast (>=
        3.6), gmp (>= 0.5-5), iterators, knitr, RColorBrewer,
        rmarkdown, testthat (>= 0.8), tikzDevice, waveslim
Description: Perform tree-ring analyses such as detrending, chronology
        building, and cross dating.  Read and write standard file formats
        used in dendrochronology.
LazyData: no
License: GPL (>= 2)
URL: https://github.com/OpenDendro/dplR
NeedsCompilation: yes
Packaged: 2026-05-20 21:26:30 UTC; andybunn
Author: Andy Bunn [aut, cph, cre, trl],
  Mikko Korpela [aut, cph, trl],
  Franco Biondi [aut, cph],
  Filipe Campelo [aut, cph],
  Stefan Klesse [aut, cph],
  Pierre Mérian [aut, cph],
  Fares Qeadan [aut, cph],
  Christian Zang [aut, cph],
  Allan Buras [ctb],
  Alice Cecile [ctb],
  Manfred Mudelsee [ctb],
  Michael Schulz [ctb],
  David Frank [ctb],
  Ronald Visser [ctb],
  Ed Cook [ctb],
  Kevin Anchukaitis [ctb]
Maintainer: Andy Bunn <bunna@wwu.edu>
Repository: CRAN
Date/Publication: 2026-05-21 05:10:20 UTC
